package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaFirstUsherRecord;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaFirstUsherRecordService;
import com.huiju.welcome.utils.DateUtils;
import com.huiju.welcome.utils.StatusUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;


/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@RestController
@RequestMapping("/")
public class TaFirstUsherRecordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaFirstUsherRecordController.class);

    @Autowired
    public ITaFirstUsherRecordService iTaFirstUsherRecordService;

    @Autowired
    private ITaCustomerService taCustomerService;

    /**
     * 保存对象
     * @param taFirstUsherRecord 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/firstUsherRecord",method= RequestMethod.POST)
    public ResponseBean save(@RequestBody TaFirstUsherRecord taFirstUsherRecord){
        ResponseBean responseBean = new ResponseBean();
        try {
            taFirstUsherRecord.setStatus(StatusUtils.Normal);

            boolean success = iTaFirstUsherRecordService.newRecordByMiniapp(taFirstUsherRecord);
            if (success){
                responseBean.addSuccess(taFirstUsherRecord);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taFirstUsherRecordAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/miniapp/firstUsherRecord/{id}/delete", method= RequestMethod.PUT)
    public ResponseBean delete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            iTaFirstUsherRecordService.removeRecordByFront(id);
            return responseBean;
        }catch (Exception e){
            logger.error("taFirstUsherRecordDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @ResponseBody
    @RequestMapping(value="/miniapp/firstUsherRecord/{id}/addwhitelist", method= RequestMethod.PUT)
    public ResponseBean addWhiteList(@PathVariable Integer id) {
        iTaFirstUsherRecordService.addCar2WhiteList(id);
        return ResponseBean.success("ok");
    }

    /**
     * 修改对象
     * @param taFirstUsherRecord 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/firstUsherRecord/{id}",method= RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaFirstUsherRecord taFirstUsherRecord, @PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {

            boolean success = iTaFirstUsherRecordService.updateByMiniapp(taFirstUsherRecord);
            if (success){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taFirstUsherRecordUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/firstUsherRecord/{id}",method= RequestMethod.GET)
    public ResponseBean get(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaFirstUsherRecordService.getById(id));
        }catch (Exception e){
            logger.error("taFirstUsherRecordGetByID -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
    
    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/miniapp/firstUsherRecord/{id}",method= RequestMethod.GET)
    public ResponseBean miniappGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            TaFirstUsherRecord taFirstUsherRecord = iTaFirstUsherRecordService.getById(id);
            TaCustomer taCustomer = taCustomerService.getCutomerByPhoneAndPlatenumber(taFirstUsherRecord.getPhone(),taFirstUsherRecord.getPlateNumber());
            if (null != taCustomer){
                taFirstUsherRecord.setCustomerName(taCustomer.getCustomerName());
                taFirstUsherRecord.setAvatar(taCustomer.getAvatar());
                taFirstUsherRecord.setCustomerId(taCustomer.getCustomerId());
            }
            responseBean.addSuccess(taFirstUsherRecord);
        }catch (Exception e){
            logger.error("taFirstUsherRecordGetByID -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
    
    /**
     * 根据id查询对象
     */
    @RequestMapping(value="/miniapp/firstUsherRecord",method= RequestMethod.GET)
    public ResponseBean list(
        @RequestParam(defaultValue = "1") int pageNumber,
        @RequestParam(defaultValue = "10") int pageSize,
        @RequestParam(defaultValue = "false") boolean today,
        @RequestParam(defaultValue = "false") boolean done,
        @RequestParam(defaultValue = "false") boolean onlyNew,
        @RequestParam(defaultValue = "0") int fromRec
    ) {
        ResponseBean responseBean = new ResponseBean();

        IPage<TaFirstUsherRecord> pg = new Page<>(pageNumber, pageSize);
        QueryWrapper<TaFirstUsherRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.ge(today, "visite_date", DateUtils.morning());
        queryWrapper.gt("rec_id", fromRec);

        if (onlyNew) {
            queryWrapper.eq("status", StatusUtils.Ready);
        } else {
            queryWrapper.ge("status", StatusUtils.Ready);
            queryWrapper.lt(!done, "status", StatusUtils.BizDone);
        }
        queryWrapper.orderByDesc("rec_id");
    
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        
        IPage<TaFirstUsherRecord> result = iTaFirstUsherRecordService.page(pg, queryWrapper);
        List<TaFirstUsherRecord> taFirstUsherRecordList = result.getRecords();
        taFirstUsherRecordList.stream().forEach(e -> {
            TaCustomer taCustomer = taCustomerService.getByPhoneOrPlate(e.getPhone(),e.getPlateNumber());
            if (null != taCustomer){
                if (formatter.format(taCustomer.getCreateDate()).equals(formatter.format(LocalDateTime.now()))){
                    e.setNewCustomer(true);
                }else {
                    e.setNewCustomer(false);
                }
            }else {
                e.setNewCustomer(true);
            }
        });
    
        result.setRecords(taFirstUsherRecordList);

//            if (onlyNew) {
//               iTaFirstUsherRecordService.updateNormal(result.getRecords());
//            }
        responseBean.addSuccess(result);
        return responseBean;
    }

    @RequestMapping(value="/firstUsherRecord",method= RequestMethod.GET)
    public ResponseBean list(@RequestParam(defaultValue = "1") int pageNum,
                             @RequestParam(defaultValue = "10") int pageSize,
                             @RequestParam(value = "plateNumber",required = false) String plateNumber,
                             @RequestParam(value = "visiteDate",required = false) String visiteDate){
            ResponseBean responseBean = new ResponseBean();
            IPage<TaFirstUsherRecord> pg = new Page<>(pageNum, pageSize);
            IPage<TaFirstUsherRecord> result = iTaFirstUsherRecordService.taFirstUsherRecordlist(pg,plateNumber,visiteDate);
            responseBean.addSuccess(result);
            return responseBean;
    }

}
