package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaTransactionRecord;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaTransactionRecordService;
import com.huiju.welcome.utils.StatusUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 成交记录表 前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-07-17
 */
@RestController
@RequestMapping("/")
public class TaTransactionRecordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaTransactionRecordController.class);

    @Autowired
    public ITaTransactionRecordService iTaTransactionRecordService;

    @Autowired
    public ITaCustomerService iTaCustomerService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taTransactionRecord",method= RequestMethod.GET)
    public ResponseBean taTransactionRecordList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                                @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaTransactionRecord> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaTransactionRecord> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaTransactionRecord> result = iTaTransactionRecordService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            logger.error("taTransactionRecordList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taTransactionRecord 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/transactionRecord",method= RequestMethod.POST)
    public ResponseBean taTransactionRecordAdd(@RequestBody TaTransactionRecord taTransactionRecord){
        ResponseBean responseBean = new ResponseBean();
        try {
            // 反更新客户表
            UpdateWrapper<TaCustomer> wrapper = new UpdateWrapper<>();
            wrapper.eq("customer_id", taTransactionRecord.getCustomerId());
            wrapper.set("status", StatusUtils.BizDone);
            iTaCustomerService.update(new TaCustomer(), wrapper);

            if (iTaTransactionRecordService.save(taTransactionRecord)){
                responseBean.addSuccess(taTransactionRecord);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taTransactionRecordAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taTransactionRecord/{id}", method= RequestMethod.DELETE)
    public ResponseBean taTransactionRecordDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaTransactionRecordService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taTransactionRecordDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taTransactionRecord 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/transactionRecord/{id}",method= RequestMethod.PUT)
    public ResponseBean taTransactionRecordUpdate(@PathVariable Integer id,
                                        @RequestBody TaTransactionRecord taTransactionRecord){
        ResponseBean responseBean = new ResponseBean();
        try {
            // 反更新客户表
            UpdateWrapper<TaCustomer> wrapper = new UpdateWrapper<>();
            wrapper.eq("customer_id", taTransactionRecord.getCustomerId());
            wrapper.set("status", StatusUtils.BizDone);
            iTaCustomerService.update(new TaCustomer(), wrapper);

            taTransactionRecord.setRecordId(id);
            if (iTaTransactionRecordService.updateById(taTransactionRecord)){
                responseBean.addSuccess(taTransactionRecord);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taTransactionRecordUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据客户id查询对象
     * @param customerId  客户ID
     */
    @RequestMapping(value="/miniapp/transactionRecord/{customerId}",method= RequestMethod.GET)
    public ResponseBean taTransactionRecordGet(@PathVariable Integer customerId){
        ResponseBean responseBean = new ResponseBean();
        try {
            TaTransactionRecord taTransactionRecord = iTaTransactionRecordService.getRecordByCustomerId(customerId);
            responseBean.addSuccess(taTransactionRecord);
        }catch (Exception e){
            logger.error("taTransactionRecordDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
