package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.SysUserMapper;
import com.huiju.welcome.mapper.TaCustomerMapMapper;
import com.huiju.welcome.mapper.TaCustomerMapper;
import com.huiju.welcome.mapper.TaFaceLibMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.*;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.net.HttpURLConnection;
import java.net.URL;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class CameraServiceImpl implements ICameraService {
    final static String HIK_FACE_LIB = "hik-face";
    final static String ALI_FACE_LIB = "ali-face";

    @Autowired
    private ITaDeviceService taDeviceService;

    @Autowired
    TencentFaceUtils tencentFaceUtils;

    @Autowired
    AliFaceUtils aliFaceUtils;

    @Autowired
    ISysParamService iSysParamService;

    @Autowired
    ITaPersonService taPersonService;

    @Autowired
    ITaSnapshotPersonService taSnapshotPersonService;

    @Autowired
    ITaVisitingLogService visitingLogService;

    @Autowired
    ITaCustomerService iTaCustomerService;
    
    @Autowired
    private TaCustomerMapMapper taCustomerMapMapper;
    
    @Autowired
    private TaCustomerMapper taCustomerMapper;
    
    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private TaFaceLibMapper taFaceLibMapper;

    @Autowired
    private IMessageService iMessageService;
    
    @Autowired
    MiniApp miniApp;

    @Override
    public String dispatchPerson(TaPerson taPerson, TaDevice... devices) {
        List<TaDevice> taDeviceList = null;
        if (null == devices || devices.length == 0) {
            QueryWrapper<TaDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("status",StatusUtils.Normal);
            queryWrapper.eq("afr",StatusUtils.BoolTrue);
            taDeviceList = taDeviceService.list(queryWrapper);
        } else {
            taDeviceList = Arrays.asList(devices);
        }

        //分发人脸到各个相机
        for(TaDevice device: taDeviceList) {
            try {
                Map<String, Object> payload = new HashMap<>();
                payload.put("personId", taPerson.getPersonId());
                payload.put("image", taPerson.getAvatar());

                Action act = Action.create(Action.TYPE_DISPATCH_FACE, payload);

                WebSocketServer.sendInfo(act.toJSON(), "device", device.getDeviceId());

                // 假设海康设备下发成功
                TaFaceLib taFaceLib = new TaFaceLib();
                taFaceLib.setLocalPerson(taPerson.getPersonId());
                taFaceLib.setLibCode(HIK_FACE_LIB);    // 海康设备
                taFaceLib.setLibPerson(taPerson.getPersonId().toString());
                taFaceLib.setLibFace(taPerson.getPersonId().toString());
                taFaceLib.setDeviceId(device.getDeviceId());
                taFaceLibMapper.insert(taFaceLib);

            } catch (Exception e){
                e.printStackTrace();
                return "连接设备(" + device.getDeviceId() + ")人脸下发服务 失败: "+ e.getMessage();
            }
        }
//
//        // 下发到腾讯云人脸识别
//        try {
//            tencentFaceUtils.createFace(String.valueOf(taPerson.getRealId()), taPerson.getAvatar());
//        } catch (TencentCloudSDKException e) {
//            e.printStackTrace();
//            log.error("下发人脸到腾讯云失败: " + e.getMessage());
//        }

        try {
            JSONObject result = aliFaceUtils.createFace(String.valueOf(taPerson.getRealId()), taPerson.getAvatar());
            if (null != result && result.getInteger("code") == HttpStatus.SC_OK) {
                JSONArray faceImageItems = result.getJSONArray("faceImageItems");
                if (null != faceImageItems && faceImageItems.size() > 0) {
                    JSONObject faceInfo = faceImageItems.getJSONObject(0);
//                    Double faceId = faceInfo.getDouble("faceId");     // 阿里返回值是一个 double 类型
                    String faceId = faceInfo.getString("faceId");

                    //
                    TaFaceLib taFaceLib = new TaFaceLib();
                    taFaceLib.setLocalPerson(taPerson.getPersonId());
                    taFaceLib.setLibCode(ALI_FACE_LIB);    // 海康设备
                    taFaceLib.setLibPerson(taPerson.getPersonId().toString());
                    taFaceLib.setLibFace(faceId);
//                    taFaceLib.setDeviceId();
                    taFaceLibMapper.insert(taFaceLib);
                }

            } else {
                log.error("下发人脸到阿里云失败: 原因未知");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("下发人脸到阿里云失败: " + e.getMessage());
        }

        return "";
    }

    @Override
    public boolean deleteFace(Integer personId) {
        QueryWrapper<TaFaceLib> wrapper = new QueryWrapper<>();
        wrapper.eq("local_person", personId);

        List<TaFaceLib> taFaceLibs = taFaceLibMapper.selectList(wrapper);
        if (null == taFaceLibs || taFaceLibs.size() == 0) {
            return true;
        }

        for (TaFaceLib taFaceLib: taFaceLibs) {
            String libCode = taFaceLib.getLibCode();

            if (libCode.indexOf(ALI_FACE_LIB) > -1) {
                // 删除阿里人脸
                try {
                    aliFaceUtils.deleteFace(taFaceLib.getLibPerson(), taFaceLib.getLibFace());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                // 删除海康人脸
                try {
                    Map<String, Object> payload = new HashMap<>();
                    payload.put("personId", personId);
                    Action act = Action.create(Action.TYPE_DELETE_FACE, payload);
                    WebSocketServer.sendInfo(act.toJSON(), "device", taFaceLib.getDeviceId());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            // 清除数据库
            taFaceLibMapper.deleteById(taFaceLib);
        }

        return false;
    }

    @Override
    public boolean newPersonByDevice(TaSnapshotPerson taSnapshotPerson) {
        boolean workingTime = iSysParamService.isInWorkingTime(LocalDateTime.now());
        if (!workingTime) {
            taSnapshotPersonService.save(taSnapshotPerson);
            return true;
        }

        // 人像质量最低值, 低于这个值的认为人脸抓拍失败, 取值范围 1 - 100
        Float faceMinScore = iSysParamService.getFaceMinScore();
        Integer score = taSnapshotPerson.getScore();
        if (null == score || score.intValue() < faceMinScore) {
            log.error("人脸质量过低, 放弃本次抓拍人脸 {}", taSnapshotPerson.getAvatar());
            return false;
        }

        // 人脸相似度阈值, 低于设定认为没有识别成功
        // SDK 中这个值范围是 0.001 - 1, 本系统设定范围是 1 - 100
        Float samePersonThreshold = iSysParamService.getSamePersonThreshold();
        Float similarity = taSnapshotPerson.getSimilarity();

        // 识别结果
        boolean newGuy = null == similarity || similarity < (samePersonThreshold / 100);
        TaPerson matchPerson = null;
        if (!newGuy && null != taSnapshotPerson.getMatchPerson()) {
            matchPerson = taPersonService.getById(taSnapshotPerson.getMatchPerson());
        }

        if (null != matchPerson) {
            LocalDateTime lastTime = matchPerson.getLastVisiteDate();
            boolean sameDevice = taSnapshotPerson.getDeviceId().equals(matchPerson.getLastVisiteDevice());

            // 降噪设定
            if (null != lastTime && sameDevice) {
                Float dispatchTimeLimit = iSysParamService.getDispatchTimeLimit();

                // 默认 2 分钟内连续识别的都是垃圾数据
                Duration duration = Duration.between(lastTime, taSnapshotPerson.getCreateDate());
                if (duration.getSeconds() < (60 * dispatchTimeLimit)) {
                    return true;
                }
            }

            // 如果是工作人员则忽略
            if (null != matchPerson && null != matchPerson.getTypeId() && matchPerson.getTypeId().equals(CommConstant.TYPE_INNER_PERSON)) {
                taSnapshotPerson.setMatchPerson(matchPerson.getRealId());
                taSnapshotPerson.setPersonId(matchPerson.getPersonId());
                taSnapshotPersonService.save(taSnapshotPerson);
                return true;
            }
        }

        // 生成新的人员
        TaPerson taPerson = taPersonService.newPersonByDevice(taSnapshotPerson, matchPerson);
        if (null == matchPerson || taPerson.getPersonId() != matchPerson.getPersonId()) {
            // 同步相机识别库
            dispatchPerson(taPerson);
        }

        //二号岗设备推送
        TaDevice taDevice = taDeviceService.getById(taSnapshotPerson.getDeviceId());
        if(!taDevice.getAfr().equals(CommConstant.DEVICE_AFR_UNFACE)){
            notifyCustomer(taPerson);

            // 预写到访日志
            visitingLogService.newLogByDevice(taSnapshotPerson, taPerson);
    
            // 判断是否客户来访
            try {
                iTaCustomerService.judgeNewCust(taPerson);
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        }else{
            // 推送消息
            taPersonService.send2Front(taPerson);
        }
    
        // 保存抓拍日志
        taSnapshotPersonService.save(taSnapshotPerson);
        
        return true;
    }

    private void notifyCustomer(TaPerson taPerson) {
        TaCustomerMap taCustomerMap = taCustomerMapMapper.getCustomerOf(taPerson.getPersonId());
        if (null != taCustomerMap){
            TaCustomer taCustomer = taCustomerMapper.selectById(taCustomerMap.getCustomerId());
            if (null != taCustomer){
                iMessageService.toggledByCustomer(taCustomer);
            }
        }
    }
}