package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.SysParamMapper;
import com.huiju.welcome.model.SysParam;
import com.huiju.welcome.service.ISysParamService;
import com.huiju.welcome.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-14
 */
@Slf4j
@Service
public class SysParamServiceImpl extends ServiceImpl<SysParamMapper, SysParam> implements ISysParamService {
    @Autowired
    SysParamMapper sysParamMapper;

    @Override
    public Float getSamePersonThreshold() {
        String val = getParamOfWithDefault("same-person-threshold", "70");
        return Float.parseFloat(val);
    }

    @Override
    public String getParamOfWithDefault(String id, String def) {
        SysParam sysParam = sysParamMapper.selectById(id);
        if (null == sysParam) return def;

        String val = sysParam.getValue();
        if (null == val || "".equals(val.trim())) return def;

        return val.trim();
    }

    @Override
    public Float getFaceMinScore() {
        String val = getParamOfWithDefault("face-min-score", "70");
        return Float.parseFloat(val);
    }

    @Override
    public Float getDispatchTimeLimit() {
        String val = getParamOfWithDefault("dispatch-time-limit", "2");
        return Float.parseFloat(val);
    }

    @Override
    public Float getTplMessageInterval() {
        String val = getParamOfWithDefault("tpl-message-interval", "120");
        return Float.parseFloat(val);
    }

    @Override
    public Float getCustomerVisitInterval() {
        String val = getParamOfWithDefault("customer-visit-interval", "120");
        return Float.parseFloat(val);
    }

    @Override
    public boolean isInWorkingTime(LocalDateTime dt) {
        String val = getParamOfWithDefault("working-time-range", "8:00-19:00");
        String[] parts = val.split("-");

        if (null == parts || parts.length != 2) {
            log.error("系统参数 working-time-range 维护不正确: {}", val);
            return false;
        }

        LocalDateTime start = DateUtils.initByHourAndMinute(parts[0]);
        LocalDateTime end = DateUtils.initByHourAndMinute(parts[1]);

        return dt.isAfter(start) && dt.isBefore(end);
    }

}
