package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.TaVisitorAppointmentMapper;
import com.huiju.welcome.model.TaVisitorAppointment;
import com.huiju.welcome.service.ITaVisitorAppointmentService;
import com.huiju.welcome.utils.DateUtils;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@Service
public class TaVisitorAppointmentServiceImpl extends ServiceImpl<TaVisitorAppointmentMapper, TaVisitorAppointment> implements ITaVisitorAppointmentService {
    @Autowired
    TaVisitorAppointmentMapper taVisitorAppointmentMapper;

    @Override
    public ResponseBean check(TaVisitorAppointment taVisitorAppointment) {
        String phone = taVisitorAppointment.getPhone();
        String platNumber = taVisitorAppointment.getPlatNumber();
        LocalDateTime visitDate = taVisitorAppointment.getVisitDate();

        List<TaVisitorAppointment> list = taVisitorAppointmentMapper.getSameAppointments(platNumber, phone, DateUtils.localDatetime2day(visitDate));
        if (null != list && list.size() > 0) {
            return ResponseBean.error("请勿对该人员进行重复预约", HttpStatus.SC_BAD_REQUEST);
        }

        return ResponseBean.success("");
    }

    @Override
    public List<TaVisitorAppointment> getAppointmentsBy(Map<String, Object> filters) {
        QueryWrapper<TaVisitorAppointment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", filters.get("phone"));
        queryWrapper.eq("status", StatusUtils.Normal);
        queryWrapper.orderByDesc("create_date");

        return taVisitorAppointmentMapper.selectList(queryWrapper);
    }

    @Override
    public List<TaVisitorAppointment> getAppointmentsByPlateNumber(String plateNumber) {
        QueryWrapper<TaVisitorAppointment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plat_number", plateNumber);
        queryWrapper.eq("status", StatusUtils.Normal);
        queryWrapper.orderByDesc("create_date");

        return taVisitorAppointmentMapper.selectList(queryWrapper);
    }

    @Override
    public IPage<TaVisitorAppointment> getMyAppointmentList(int pageNum, int pageSize, Integer personId, String nameOrPhone, String identity) {
        IPage<TaVisitorAppointment> page = new Page<>(pageNum, pageSize);
        IPage<TaVisitorAppointment> result = taVisitorAppointmentMapper.getMyAppointmentList(page, personId, nameOrPhone, identity);

        return result;
    }
    
    @Override
    public IPage<TaVisitorAppointment> getTodayAppointmentList(int pageNum, int pageSize) {
        IPage<TaVisitorAppointment> page = new Page<>(pageNum, pageSize);
        IPage<TaVisitorAppointment> result = taVisitorAppointmentMapper.getTodayAppointmentList(page);
    
        return result;
    }
    
    @Override
    public ResponseBean miniappVisitorAppointmentList(IPage<TaVisitorAppointment> pg, String platNumber, String personName, String phone) {
        ResponseBean responseBean= new ResponseBean();
        IPage<TaVisitorAppointment> list= taVisitorAppointmentMapper.miniappVisitorAppointmentList(pg,platNumber,personName,phone);
        responseBean.addSuccess(list);
        return responseBean;
    }
    
    @Override
    public TaVisitorAppointment bindAppointmentsBy(Integer appointmentId, String platNumber) {
        TaVisitorAppointment appointment = null;

        if (null != appointmentId && appointmentId > 0) {
            appointment = taVisitorAppointmentMapper.selectById(appointmentId);
        }

        if (appointment == null && !StringUtils.isEmpty(platNumber)) {
            QueryWrapper<TaVisitorAppointment> query = new QueryWrapper<>();
            query.eq("plat_number", platNumber);
            query.gt("status", StatusUtils.Delete);
            query.lt("status", StatusUtils.BizDone);
            query.last(" order by IFNULL(visit_date, create_date) asc  limit 1");
            appointment = taVisitorAppointmentMapper.selectOne(query);
        }

        if (null != appointment) {
            appointment.setStatus(StatusUtils.BizDone);
            taVisitorAppointmentMapper.updateById(appointment);
        }

        return appointment;
    }

    @Override
    public ResponseBean visitorAppointmentList(IPage pg, String platNumber, String personName,String phone) {
       ResponseBean responseBean= new ResponseBean();
        IPage<TaVisitorAppointment> list= taVisitorAppointmentMapper.visitorAppointmentList(pg,platNumber,personName,phone);
        responseBean.addSuccess(list);
        return responseBean;
    }
}
